# core-image-publisher
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import argparse
import configparser
import logging
import os

import logstash

from core_image_publisher.queue import (
    create_connection_from_config,
    CoreImagePublisherQueueMonitor,
)
from core_image_publisher import worker

logger = logging.getLogger(__name__)


def read_config():
    parser = argparse.ArgumentParser(description='Core image publisher.')
    parser.add_argument(
        '-c',
        '--conf',
        default='core-service.conf',
        help='Configuration file path'
    )
    args = parser.parse_args()

    # Load configuration options.
    config = configparser.ConfigParser()
    config.read(args.conf)
    return config


def configure_logging(config):
    root_logger = logging.getLogger()
    root_logger.setLevel(logging.INFO)

    # If there is no ./logs directory, fallback to stderr.
    log_path = os.path.abspath(
        os.path.join(__file__, '../../logs/core-image-publisher.log'))
    log_dir = os.path.dirname(log_path)
    if os.path.exists(log_dir):
        handler = logging.FileHandler(log_path)
    else:
        print("'logs' directory '{}' does not exist, using stderr "
              "for app log.".format(log_dir))
        handler = logging.StreamHandler()

    handler.setFormatter(
        logging.Formatter(
            '%(asctime)s  %(name)s %(levelname)s: %(message)s'
        )
    )
    root_logger.addHandler(handler)
    if 'logstash' in config:
        root_logger.addHandler(
            logstash.LogstashHandler(
                config['logstash']['host'],
                int(config['logstash']['port']),
                int(config['logstash']['version'])
            )
        )


def main():
    config = read_config()
    configure_logging(config)
    connection = create_connection_from_config(config)
    try:
        with connection:
            monitor = CoreImagePublisherQueueMonitor(
                connection,
                worker.logging_worker
            )
            monitor.run()
    except KeyboardInterrupt:
        print("Bye!")
