# core-image-publisher
# Copyright (C) 2015 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import logging

import kombu
from kombu.mixins import ConsumerMixin

from core_image_publisher import constants

logger = logging.getLogger(__name__)

__all__ = [
    'CoreImagePublisherQueueMonitor',
    'create_connection_from_config',
    'enqueue_message',
]


class CoreImagePublisherQueueMonitor(ConsumerMixin):

    """A class that watches for incoming messages from a queue, and calls
    a callback to process them.

    By using an external callback we can separate the stateful logic of
    connecting to amqp from our business logic of dealing with the messages.

    """

    def __init__(self, connection, on_message_cb):
        """Construct a new CoreImagePublisherQueueMonitor.

        :param connection: A kombu.Connection instance.
        :param on_message_db: A callable that will be called for every new
            messages from the rabbitMQ queue(s). It must accept a single
            parameter: the message object. The callback MUST either acknowledge
            or reject or requeue the messsage, and MUST NOT raise any unhandled
            exceptions.

        """
        self.connection = connection
        self.on_message_cb = on_message_cb

    def get_consumers(self, Consumer, channel):
        """Return consumers instances for all configured queues."""
        queues = [
            kombu.Queue('core.image.{}'.format(constants.API_VERSION))
        ]
        return [Consumer(queues=queues, callbacks=[self.process])]

    def process(self, body, message):
        """Process incomming test request.

        Run requested tests and posts results to the 'adt_results' queue
        for later checking.
        """
        logger.info('Got: %r', body, extra=body)
        try:
            self.on_message_cb(message)
        except Exception as e:
            logger.error(
                "Caught unhandled exception while processing message: %s",
                e,
                extra=message.payload
            )
            message.requeue()


def create_connection_from_config(config):
    """Create a connection given a config object.

    'config' can either be a ConfigParser instance, or a nested dictionary.

    """
    amqp_uris = config.get('amqp', 'uris').split()
    return kombu.Connection(amqp_uris)


def enqueue_message(payload):
    """Enqueue 'payload' for the core-image-tester."""
